import json
import requests
import logging
from r360_py.util.Configuration import Configuration


class ServiceExecutor:
    'This class should be used to query the service'

    def __init__(self):
        return

    def execute_service(self, travel_options, service):

        params = {
            "key": travel_options.getServiceKey(),
            "cfg": json.dumps(Configuration.build(travel_options))
        }

        r = requests.get(travel_options.getServiceUrl() + 'v1/' + service, params=params)

        if r.status_code == 403:
            raise PermissionError(r.text)
        elif r.status_code != 200:
            logging.getLogger("Service").error(r.text)
        else:
            if service == "route":
                json_body = r.text.replace("null(", "").rstrip(")")
                return json.loads(json_body)
            else:
                return json.loads(r.text)



from enum import Enum

class PolygonIntersectionMode(Enum):
    UNION = "union"
    INTERSECTION = "intersection"
    AVERAGE = "average"

    @classmethod
    def parse(clazz, str):
        
        if str is None:
            raise Exception('PolygonIntersectionMode: "None" not supported!')

        type = getattr(clazz, str.upper(), None)
        if type is None:
            raise Exception('PolygonIntersectionMode: "' + str + '" not supported!')

        return type


from r360_py.rest.ServiceExecutor import ServiceExecutor

if __name__ == '__main__':
    import json
    import argparse
    from r360_py.util.TravelOptions import TravelOptions
    from r360_py.util.enum.PolygonSerializationType import PolygonSerializationType
    from r360_py.util.enum.TravelType import TravelType

    def source(arg):
        # For simplity, assume arg is a pair of integers
        # separated by a comma. If you want to do more
        # validation, raise argparse.ArgumentError if you
        # encounter a problem.
        return [float(x) for x in arg.split(';')]

    parser = argparse.ArgumentParser(prog="python -m r360_py.cli", description="Query the Route360 Polygon service using python", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--time",              type=int,    help="The time in seconds of the day: 1.30 p.m. = 13 * 3600 + 30 * 60 = 48600 (transit only)", default=43200)
    parser.add_argument("--date",              type=int,    help="The date in the format YYYYMMDD, e.g.: 20160727 for the 27th of July 2016 (transit only)", default=20162707)
    parser.add_argument("--polygonSerializer", type=str,    help="The serializer for the polygons: json or geojson", default="geojson")
    parser.add_argument("--buffer",            type=float,  help="The buffer (in srid units) that should be generated around the polygons.", default=None)
    parser.add_argument("--simplify",          type=int,    help="The threshold (in meter) that should be used for Douglas-Puecker (before buffering, max 500m).", default=None)
    parser.add_argument("--srid",              type=int,    help="The target SRID (Spatial Reference System Identifier), all that are supported via PostGIS.", default=None)
    parser.add_argument("--quadrantSegments",  type=int,    help="The number of quadrant segements (max 8), see: http://postgis.net/docs/ST_Buffer.html.", default=None)

    requiredNamed = parser.add_argument_group('required named arguments')
    requiredNamed.add_argument("--travelTimes",       type=int,    help="The travel time in seconds as a list of integers.", nargs="+", required=True)
    requiredNamed.add_argument("--serviceUrl",        type=str,    help="The URL of the Route360° API endpoint.", required=True)
    requiredNamed.add_argument("--serviceKey",        type=str,    help="Your personal key for the API.", required=True)
    requiredNamed.add_argument("--travelType",        type=str,    help="The travel type for the request: car, walk, bike or transit", required=True)
    requiredNamed.add_argument("--source",            type=source, help="The source as doubles (lat,lng) separated by ';'.", required=True)
    requiredNamed.add_argument("--outputDir",         type=str,    help="The path where to write the output files", required=True)
    requiredNamed.add_argument("--outputFilename",    type=str,    help="The the name of the file to write to", required=True)

    args = parser.parse_args()

    travelOptions = TravelOptions();
    travelOptions.addSource({ "id": str(args.source[0]) + ";" + str(args.source[1]), "lat" :  args.source[0],  "lng" :  args.source[1], "tm" : {  args.travelType : {
        "date" : args.date, "time" : args.time
    }}})
    travelOptions.setServiceKey(args.serviceKey)
    travelOptions.setTravelTimes(args.travelTimes)
    travelOptions.setServiceUrl(args.serviceUrl)
    travelOptions.setTravelDate(args.date)
    travelOptions.setTravelTime(args.time)
    travelOptions.setBuffer(args.buffer)
    travelOptions.setSimplifyMeter(args.simplify)
    travelOptions.setSrid(args.srid)
    travelOptions.setQuadrantSegments(args.quadrantSegments)
    travelOptions.setTravelType(TravelType.parse(args.travelType))
    travelOptions.setPolygonSerializationType(PolygonSerializationType.parse(args.polygonSerializer))

    polygon = ServiceExecutor().execute_service(travelOptions, "polygon")

    f = open(args.outputDir + args.outputFilename, 'w')
    f.write(json.dumps(polygon["data"]))



from distutils.core import setup
from setuptools import find_packages
setup(
  name = 'r360_py',
  packages = find_packages(),
  version = '0.12',
  description = 'A python client library to query the Route360° API',
  author = 'Motion Intelligence GmbH',
  author_email = 'mail@motionintelligence.net',
  url = 'https://github.com/route360/r360-py',
  download_url = 'https://github.com/route360/r360-py/tarball/0.12',
  install_requires=[
     'requests',
 ],
  keywords = ['isochrone', 'routing', 'polygon', 'openstreetmaps', 'gtfs', 'map'],
  classifiers = [],
)



from r360_py.util.Util import Util
from r360_py.util.enum.EdgeWeightType import EdgeWeightType
from r360_py.util.enum.PathSerializerType import PathSerializerType
from r360_py.util.enum.PolygonIntersectionMode import PolygonIntersectionMode

class TravelOptions:
    'This class represents the configuration for one request'

    def __init__(self):
        self.sources = []
        self.targets = []
        self.travelTimes = []
        self.travelType = None
        self.buffer = None
        self.simplifyMeter = None
        self.quadrantSegments = None
        self.srid = None
        self.travelDate = Util.currentDate()
        self.travelTime = Util.currentTime()
        self.minPolygonHoleSize = 100000000
        self.intersectionMode = PolygonIntersectionMode.UNION
        self.polygonSerializationType = None
        self.pathSerializer = PathSerializerType.COMPACT_PATH_SERIALIZER
        self.edgeWeightType = EdgeWeightType.TIME

    def setPolygonSerializationType(self, polygonSerializationType):
        self.polygonSerializationType = polygonSerializationType

    def getPolygonSerializationType(self):
        return self.polygonSerializationType

    def setServiceUrl(self, serviceUrl):
        self.serviceUrl = serviceUrl

    def getServiceUrl(self):
        return self.serviceUrl

    def setServiceKey(self, serviceKey):
        self.serviceKey = serviceKey

    def getServiceKey(self):
        return self.serviceKey

    def setSources(self, sources):
        self.sources = sources

    def getSources(self):
        return self.sources

    def addSource(self, source):
        self.sources.append(source)

    def setTargets(self, targets):
        self.targets = targets

    def getTargets(self):
        return self.targets

    def addTarget(self, target):
        self.targets.append(target)

    def setTravelTimes(self, travelTimes):
        self.travelTimes = travelTimes;

    def getTravelTimes(self):
        return self.travelTimes;

    def addTravelTime(self, travelTime):
        self.travelTimes.append(source)

    def setMinPolygonHoleSize(self, minPolygonHoleSize):
        self.minPolygonHoleSize = minPolygonHoleSize

    def getMinPolygonHoleSize(self):
        return self.minPolygonHoleSize

    def setTravelType(self, travelType):
        self.travelType = travelType

    def getTravelType(self):
        return self.travelType

    def setTravelTime(self, travelTime):
        self.travelTime = travelTime

    def getTravelTime(self):
        return self.travelTime

    def setTravelDate(self, travelDate):
        self.travelDate = travelDate

    def getTravelDate(self):
        return self.travelDate

    def setPolygonIntersectionMode(self, intersectionMode):
        self.intersectionMode = intersectionMode

    def getPolygonIntersectionMode(self):
        return self.intersectionMode

    def setBuffer(self, buffer):
        self.buffer = buffer

    def getBuffer(self):
        return self.buffer

    def setSimplifyMeter(self, simplifyMeter):
        self.simplifyMeter = simplifyMeter

    def getSimplifyMeter(self):
        return self.simplifyMeter

    def setQuadrantSegments(self, quadrantSegments):
        self.quadrantSegments = quadrantSegments

    def getQuadrantSegments(self):
        return self.quadrantSegments

    def setSrid(self, srid):
        self.srid = srid

    def getSrid(self):
        return self.srid

    def setPathSerializer(self, path_serializer):
        self.pathSerializer = path_serializer

    def getPathSerializer(self):
        return self.pathSerializer

    def getEdgeWeightType(self):
        return self.edgeWeightType

    def setEdgeWeightType(self, edge_weight_type):
        self.edgeWeightType = edge_weight_type
